/*
** image_effects.c
** 
** Made by (Arne Caspari)
** Login   <arne@arne-laptop>
** 
** Started on  Wed Feb  7 10:35:35 2007 Arne Caspari
** Last update Thu Feb 15 18:07:28 2007 Arne Caspari
*/

/*
  Copyright (C) 2007  Arne Caspari <arne@unicap-imaging.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <unicap.h>
#include <unicapgtk.h>
#include <ucil.h>
#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <math.h>

#include <time.h>

#include "image_effects.h"


static ucil_font_object_t *fobj;



void ifx_draw_line( unicap_data_buffer_t *buffer, gint *points )
{
   ucil_color_t scol, dcol;

   scol.colorspace = UCIL_COLORSPACE_RGB24;
   scol.rgb24.r = 0xff;
   scol.rgb24.g = 0xff;
   scol.rgb24.b = 0xff;
   
   dcol.colorspace = ucil_get_colorspace_from_fourcc( buffer->format.fourcc );
   ucil_convert_color( &dcol, &scol );

   ucil_draw_line( buffer, &dcol, points[0], points[1], points[2], points[3] );
}

void ifx_draw_measure( unicap_data_buffer_t *buffer, gint *points )
{
   ucil_color_t scol, dcol;
   gint x1, x2, y1, y2;
   gint a,b;
   double c;
   char text[128];
   int w,h;
   int textx, texty;

   scol.colorspace = UCIL_COLORSPACE_RGB24;
   scol.rgb24.r = 0xff;
   scol.rgb24.g = 0xff;
   scol.rgb24.b = 0xff;
   
   dcol.colorspace = ucil_get_colorspace_from_fourcc( buffer->format.fourcc );
   ucil_convert_color( &dcol, &scol );

   x1 = points[0];
   y1 = points[1];
   x2 = points[2];
   y2 = points[3];

   x1 = MIN( buffer->format.size.width, x1 );
   x1 = MAX( 0, x1 );
   x2 = MIN( buffer->format.size.width, x2 );
   x2 = MAX( 0, x2 );
   y1 = MIN( buffer->format.size.height, y1 );
   y1 = MAX( 0, y1 );
   y2 = MIN( buffer->format.size.height, y2 );
   y2 = MAX( 0, y2 );
   
   a = abs( x1-x2 );
   b = abs( y1-y2 );
   c = sqrt( a*a + b*b );
   sprintf( text, "%.1fpx", c );

   ucil_text_get_size( fobj, text, &w, &h );

   textx = (x1 + x2) / 2;
   if( ( textx + w ) > ( buffer->format.size.width ) )
   {
      textx -= w + 10;
   }
   if( ( y1 <= y2 ) || ( x1 >= x2 ) )
   {
      texty = ( ( y1 + y2 ) / 2 ) - ( h + 10 );
   }
   else
   {
      texty = ( ( y1 + y2 ) / 2 ) + 10;
   }

   if( texty < 0 )
   {
      texty = ( ( y1 + y2 ) / 2 ) + 10;
   }
   
   ucil_draw_text( buffer, &dcol, fobj, text, textx, texty );

   ucil_draw_line( buffer, &dcol, x1, y1, x2, y2 );   
}


GtkWidget *ifx_create_color_mag_window( GtkWindow *parent, gint cols, gint rows, color_mag_window_data_t *data )
{
   GtkWidget *window;
   GtkWidget *display;

   gint text_w;
   gint text_h;

   gint box_w, box_h;
   const gint vspacing = 5;
   const gint hspacing = 5;
   
   window = gtk_window_new( GTK_WINDOW_TOPLEVEL );

   ucil_text_get_size( data->fobj, "c: 123", &text_w, &text_h );

   box_w = text_w + 2 * hspacing;
   box_h = text_h * 3 + 5 * vspacing;

   display = unicapgtk_video_display_new();
   gtk_container_add( GTK_CONTAINER( window ), display );
   gtk_widget_set_size_request( display, box_w * cols, box_h * rows );

   if( parent )
   {
      gtk_window_set_transient_for( GTK_WINDOW( window ), parent );
   }
   
   data->text_w = text_w;
   data->text_h = text_h;
   data->hspacing = hspacing;
   data->vspacing = vspacing;
   data->display = display;

   return window;
}

void ifx_update_color_mag( unicap_data_buffer_t *buffer, color_mag_window_data_t *data )
{
   static gint rows = 8;
   static gint cols = 8;
   
   gint x = data->cursor_pos[0] < ( cols/2 + 1) ? (cols/2 + 1) : 
      data->cursor_pos[0] > ( buffer->format.size.width - ( cols / 2 + 2 ) ) ? ( buffer->format.size.width - ( cols / 2 + 2 ) ) : 
      data->cursor_pos[0];

   gint y = data->cursor_pos[1] < ( rows/2 + 1) ? (rows/2 + 1) : 
      data->cursor_pos[1] > ( buffer->format.size.height - ( rows / 2 + 2 ) ) ? ( buffer->format.size.height - ( rows / 2 + 2 ) ) : 
      data->cursor_pos[1];

   ucil_color_t scol;
   ucil_color_t dcol;

   int i,j;
   
   scol.colorspace = UCIL_COLORSPACE_RGB24;
   scol.rgb24.r = 0xff;
   scol.rgb24.g = 0xff;
   scol.rgb24.b = 0xff;
   
   dcol.colorspace = ucil_get_colorspace_from_fourcc( buffer->format.fourcc );
   ucil_convert_color( &dcol, &scol );
   
   ucil_draw_box( buffer, &dcol, x - (cols/2 + 1), y - (rows/2 + 1), x + (cols/2 + 1), y + (rows/2 + 1) );


   for( i = 0; i < cols; i++ )
   {
      for( j = 0; j < rows; j++ )
      {
	 ucil_get_pixel( buffer, &data->colors[i][j], x - (cols/2-1) + i, y - (rows/2-1) + j );
      }
   }
}



void ifx_init( void )
{
   fobj = ucil_create_font_object( 14, NULL );
}


static gint ifx_compare_func( image_effect_data_t *a, image_effect_data_t *b )
{
   return strcmp( a->name, b->name );
}

image_effect_data_t *ifx_list_put( GList **list, image_effect_data_t *data )
{
   GList *entry;
   image_effect_data_t *old_data;
   old_data = NULL;
   
   if( ( entry = g_list_find_custom( *list, data, (GCompareFunc) ifx_compare_func ) ) )
   {
      old_data = entry->data;
      entry->data = data;
   }
   else
   {
      *list = g_list_append( *list, data );
   }
   
   return old_data;
}

image_effect_data_t *ifx_list_remove( GList **list, gchar *name )
{
   image_effect_data_t tmp_data;
   image_effect_data_t *old_data = NULL;
   GList *entry;
   
   tmp_data.name = name;

   if( ( entry = g_list_find_custom( *list, &tmp_data, (GCompareFunc) ifx_compare_func ) ) )
   {
      old_data = entry->data;
      *list = g_list_remove_link( *list, entry );
      g_list_free_1( entry );
   }
   else
   {
      g_warning( "ifx_list_remove: '%s' not found!\n", name );
   }
   
   return old_data;
}

      
   
   
